
#include "cuda_runtime.h"
#include "device_launch_parameters.h"

#include <stdio.h>
#include "iostream"
#include "fstream"
#include <sstream>

#include <cstdlib>
#include <ctime>
#include <string>

#include <math.h>



using namespace std;


string PATH = "C:\\Users\\M J\\Documents\\Visual Studio 2015\\Projects\\Deep\\Debug\\";




class Tools
{
public:

	/********* Boltzmann Machine Data Generator **************************/

	static float* weights;

	static float *biases;

	static int* v;

	static int Nv;

	static float maxprob;

	static void writevIntoFile(ofstream &ofs)
	{
		for (int i = 0; i < (Nv - 1); i++)
			ofs << *(v + i) << " ";
		ofs << *(v + Nv - 1) << endl;
	}

	static void zeroV(int i) { (*(v + i)) = 0; }

	static void zeroV()
	{
		for (int i = 0; i < Nv; i++)
			zeroV(i);
	}

	static void step1(int i)
	{
		(*(v + i)) = ((*(v + i) + 1) % 2);
		if (((*(v + i)) == 0) && (i < (pow(2,Nv)-1)))
			step1(i + 1);
	}

	static void step1()
	{
		step1(0);
	}

	static void genV(int i)
	{
		if (rand01() < 0.5) (*(v + i)) = 0;
		else (*(v + i)) = 1;
	}

	static void genV()
	{
		for (int i = 0; i < Nv; i++)
			genV(i);
	}

	static float energy()
	{
		float ret = 0.0;
		int k = 0;
		for (int i = 0; i < (Nv - 1); i++)
			for (int j = (i + 1); j < Nv; j++)
			{
				ret = ret - (*(v + i))*(*(weights + k))*(*(v + j));
				k++;
			}
		for (int i = 0; i < Nv; i++)
			ret = ret - biases[i] * (*(v + i));
		return ret;
	}

	static float minimum_energy()
	{
		zeroV();
		float ret = energy();
		for (int i = 1; i < (pow(2,Nv)); i++)
		{
			step1();
			float en = energy();
			if (en < ret)
				ret = en;
			if ((i % 10000) == 0) cout << "i: " << i << ", energy: " << ret << endl;
		}
		return ret;

	}

	static void setWeights(int Nvv)
	{
		Nv = Nvv;
		v = new int[Nv];
		weights = new float[(Nv*(Nv - 1)) / 2];
		biases = new float[Nv];
		for (int i = 0; i < ((Nv*(Nv - 1)) / 2); i++)
		{
			weights[i] = gaussian(0.0, sqrt(pow(gaussian(0.0, 0.5), 2.0)));
			cout << "weight[" << i << "]: " << weights[i] << endl;
		}
		for (int i = 0; i < Nv; i++)
		{
			biases[i] = rand01();
			cout << "bias[" << i << "]: " << biases[i] << endl;
		}
		maxprob = expf(-minimum_energy());
	}

	static void generateEnergyBasedDataset(int Nvv, int N, string filename)
	{
		ofstream ofs(filename);
		setWeights(Nvv);
		for (int i = 0; i < N; i++)
		{
			bool finish = false;
			int steps = 0;
			while (finish == false)
			{
				genV();
				if (rand01() < ((expf(-energy()) / maxprob)))
					finish = true;
				steps++;
			}
			if ((i % 1000) == 0) cout << "i: " << i << ", steps: " << steps << endl;
			writevIntoFile(ofs);
		}
	}

	/*****************************************************************************************/




	/********************* Random Numbers Generators ****************************************/

	static float rand01() { return (float)((double)(rand())) / ((double)(RAND_MAX)); }

	static int randInt(int MAX) 
	{
		int ret = ((MAX * rand()) / RAND_MAX); 
		if (ret == MAX) ret = ret - 1;
		return ret;
	}

	static float gaussian(float mean, float std)
	{
		float standard_gaussian = 10.0e51;
		while ((standard_gaussian != standard_gaussian) || (standard_gaussian>10.0e50) || (standard_gaussian < -10.0e50))
		{
			float r1 = rand01();
			float r2 = rand01();

			standard_gaussian = (float)(sqrt(-log(r1))*cos(2.0*3.141592653589793*r2));
		}

		return (float)(standard_gaussian*std + mean);
	}

	static void setRandomValues(float *table, int i, int type_of_randomness)
	{
		table[i] = 0.0;
		if (type_of_randomness == 1)
			if (Tools::rand01() < 0.5)
				table[i] = 1.0;
		if (type_of_randomness == 2)
			table[i] = Tools::gaussian(0.0, 1.0);
		if (type_of_randomness == 3)
			table[i] = Tools::rand01();
	}

	static void setRandomValues(int size, float *table, int type_of_randomness)
	{
		for (int i = 0; i < size; i++)
		{
			setRandomValues(table, i, type_of_randomness);
		}
	}

	/***************************************************************************************/




	/*************** String operations ****************************************************/

	static int size(string temp)
	{
		int n = 0;
		string t;
		stringstream ss(temp);
		while (getline(ss, t, ' '))
		{
			n++;
		}
		return n;
	}

	static string* splitString(string temp)
	{
		string* ret;
		int n = size(temp);
		string t;
		ret = new string[n];
		stringstream ss(temp);
		for (int i = 0; i < n; i++)
		{
			getline(ss, t, ' ');
			ret[i] = t;
		}
		return ret;
	}

	static string fromInt(int i)
	{
		if (i == 0)
			return "0";
		if (i == 1)
			return "1";
		if (i == 2)
			return "2";
		if (i == 3)
			return "3";
		if (i == 4)
			return "4";
		if (i == 5)
			return "5";
		if (i == 6)
			return "6";
		if (i == 7)
			return "7";
		if (i == 8)
			return "8";
		if (i == 9)
			return "9";
		return 0;
	}

	static string addNumbersToFilename(int* numbers, int nnum, string filename)
	{
		stringstream ss(filename);
		string s1;
		string s2;
		getline(ss, s1, '.');
		getline(ss, s2, '.');
		for (int i = 0; i < nnum; i++)
			s1 = s1 + fromInt(numbers[i]);
		s1 = s1 + "." + s2;
		return s1;
	}

	/*************************************************************************************/

	static float proportion(int N1, int N2, int i) { return ((float)(i - N1)) / ((float)(N2 - N1)); }

	static int getIntFromHexadecimals(int bytes, unsigned char* ch)
	{
		int ret = 0;
		for (int i = 0; i < bytes; i++)
			ret = ret + ((int)pow(256.0, bytes - 1 - i)) * ((int)(ch[i]));			
		return ret;
	}

	static bool isLabInSet(unsigned char lab, int *numbers, int nnum)
	{
		for (int i = 0; i < nnum; i++)
			if (numbers[i] == ((int)(lab)))
				return true;
		return false;
	}

	static void splitIntIntoChars(int I, int siz, unsigned char* ch)
	{
		int II = I;
		for (int i = 0; i < siz; i++)
		{
			(*(ch + siz - 1 - i)) = (unsigned char)(II % 256);
			II = II / 256;
		}
	}

	static int mergeHexadecimals(unsigned char i1, unsigned char i2, unsigned char i3, unsigned char i4)
	{
		return ((int)i1 * 256 + (int)i2) * 256 * 256 + (int)i3 * 256 + (int)i4;
	}


	/******************* Binary files processing *******************************/

	static int getIntFromBytes(int bytes, ifstream &file)
	{
		unsigned char *ch = new unsigned char[bytes];
		file.read((char*)ch, bytes);
		return getIntFromHexadecimals(bytes, ch);
	}

	static int scan_binaryfile(int *numbers, int nnum, string filename)
	{
		unsigned char *ch;
		unsigned char chlab;

		ifstream file(PATH+filename, ios::in | ios::binary);

		ch = new unsigned char[4];
		file.read((char*)ch, sizeof(char) * 4);
		file.read((char*)ch, sizeof(char) * 4);


		int Nsize = Tools::getIntFromHexadecimals(4, ch);
		int n = 0;

		for (int i = 0; i < Nsize; i++)
		{
			file.read((char*)&chlab, sizeof(char));
			if (Tools::isLabInSet(chlab, numbers, nnum))
				n++;
		}

		file.close();

		return n;
	}

	static void splitFileMnist(int* numbers, int nnum, string images_filename, string labels_filename)
	{
		unsigned char *ch;

		int NewNsize = Tools::scan_binaryfile(numbers, nnum, labels_filename);

		ifstream img(PATH + images_filename, ios::in | ios::binary);
		ifstream lab(PATH + labels_filename, ios::in | ios::binary);

		string outfimg = Tools::addNumbersToFilename(numbers, nnum, images_filename);
		string outflab = Tools::addNumbersToFilename(numbers, nnum, labels_filename);

		ofstream imgo(PATH + outfimg, ios::out | ios::binary);
		ofstream labo(PATH + outflab, ios::out | ios::binary);

		ch = new unsigned char[4];

		lab.read((char*)ch, sizeof(char) * 4);
		labo.write((char*)ch, sizeof(char) * 4);
		img.read((char*)ch, sizeof(char) * 4);
		imgo.write((char*)ch, sizeof(char) * 4);


		img.read((char*)ch, sizeof(char) * 4);
		lab.read((char*)ch, sizeof(char) * 4);
		int Nsize = Tools::getIntFromHexadecimals(4, ch);
		Tools::splitIntIntoChars(NewNsize, 4, ch);
		imgo.write((char*)ch, sizeof(char) * 4);
		labo.write((char*)ch, sizeof(char) * 4);


		img.read((char*)ch, sizeof(char) * 4);
		int d1 = Tools::getIntFromHexadecimals(4, ch);
		imgo.write((char*)ch, sizeof(char) * 4);
		img.read((char*)ch, sizeof(char) * 4);
		int d2 = Tools::getIntFromHexadecimals(4, ch);
		imgo.write((char*)ch, sizeof(char) * 4);

		int Dsize = d1*d2;

		ch = new unsigned char[Dsize];

		unsigned char chlab;

		for (int i = 0; i < Nsize; i++)
		{
			img.read((char*)ch, sizeof(char) * Dsize);
			lab.read((char*)&chlab, sizeof(char));

			if (Tools::isLabInSet(chlab, numbers, nnum))
			{
				imgo.write((char*)ch, Dsize * sizeof(char));
				labo.write((char*)&chlab, sizeof(char));
			}
			if ((i % 1000) == 0)
				cout << i << endl;
		}

		img.close();
		lab.close();
		imgo.close();
		labo.close();
	}

	static void makeCDFileMnist(string f1, string lf1, string f2, string lf2, string of, string lof, int Ntot, int Nchange1, int Nchange2)
	{
		unsigned char* ch;
		unsigned char chlab;

		ifstream file1(PATH + f1, ios::in | ios::binary);
		ifstream filelab1(PATH + lf1, ios::in | ios::binary);

		ifstream file2(PATH + f2, ios::in | ios::binary);
		ifstream filelab2(PATH + lf2, ios::in | ios::binary);

		ofstream ofile(PATH + of, ios::out | ios::binary);
		ofstream ofilelab(PATH + lof, ios::out | ios::binary);

		ch = new unsigned char[4];

		file1.read((char*)ch, sizeof(char) * 4);
		file2.read((char*)ch, sizeof(char) * 4);
		ofile.write((char*)ch, sizeof(char) * 4);
		filelab1.read((char*)ch, sizeof(char) * 4);
		filelab2.read((char*)ch, sizeof(char) * 4);
		ofilelab.write((char*)ch, sizeof(char) * 4);

		file1.read((char*)ch, sizeof(char) * 4);
		filelab1.read((char*)ch, sizeof(char) * 4);
		int N1 = Tools::getIntFromHexadecimals(4, ch);

		file2.read((char*)ch, sizeof(char) * 4);
		filelab2.read((char*)ch, sizeof(char) * 4);
		int N2 = Tools::getIntFromHexadecimals(4, ch);

		Tools::splitIntIntoChars(Ntot, 4, ch);

		ofile.write((char*)ch, sizeof(char) * 4);
		ofilelab.write((char*)ch, sizeof(char) * 4);

		file1.read((char*)ch, sizeof(char) * 4);
		file2.read((char*)ch, sizeof(char) * 4);
		ofile.write((char*)ch, sizeof(char) * 4);

		int d1 = Tools::getIntFromHexadecimals(4, ch);

		file1.read((char*)ch, sizeof(char) * 4);
		file2.read((char*)ch, sizeof(char) * 4);
		ofile.write((char*)ch, sizeof(char) * 4);

		int d2 = Tools::getIntFromHexadecimals(4, ch);

		int Dsize = d1*d2;

		ch = new unsigned char[Dsize];
		bool change = false;

		for (int i = 0; i < Ntot; i++)
		{
			if (i <= Nchange1) change = false;
			else if (i > Nchange2) change = true;
			else if (Tools::rand01() < Tools::proportion(Nchange1, Nchange2, i)) change = true;
			else change = false;
			if (change)
			{
				file1.read((char*)ch, sizeof(char)*Dsize);
				filelab1.read((char*)&chlab, sizeof(char));
			}
			else
			{
				file2.read((char*)ch, sizeof(char)*Dsize);
				filelab2.read((char*)&chlab, sizeof(char));
			}

			ofile.write((char*)ch, sizeof(char)*Dsize);
			ofilelab.write((char*)&chlab, sizeof(char));
		}

		file1.close();
		file2.close();
		filelab1.close();
		filelab2.close();
		ofile.close();
		ofilelab.close();

	}

	static void makeCDFile(string f1, string f2, string of, int Ntot, int Nchange1, int Nchange2, bool mnst)
	{
		if (!mnst)
		{
			string temp;

			ifstream if1(PATH + f1);
			ifstream if2(PATH + f2);

			ofstream ofile(PATH + of);

			bool change;
			for (int i = 0; i < Ntot; i++)
			{
				if (i <= Nchange1) change = false;
				else if (i > Nchange2) change = true;
				else if (Tools::rand01() < Tools::proportion(Nchange1, Nchange2, i)) change = true;
				else change = false;
				if (change)
				{
					getline(if2, temp);
					ofile << temp << endl;
				}
				else
				{
					getline(if1, temp);
					ofile << temp << endl;
				}
			}

			ofile.close();;
			if1.close();
			if2.close();
		}
		else
		{
			stringstream ss1(f1);
			string s1;
			getline(ss1, s1, '.');
			s1 = s1 + "." + "idx1-ubyte";
			stringstream ss2(f2);
			string s2;
			getline(ss2, s2, '.');
			s2 = s2 + "." + "idx1-ubyte";
			stringstream oss(of);
			string os;
			getline(oss, os, '.');
			os = os + "." + "idx1-ubyte";
			makeCDFileMnist(f1, s1, f2, s2, of, os, Ntot, Nchange1, Nchange2);
		}
	}

	/********************************************************************************************************************/

	static void makeOneFileAssigningClasses(string f1, string f2, string of, int Ntot)
	{
		string temp;

		ifstream if1(PATH+f1);
		ifstream if2(PATH+f2);

		ofstream ofile(PATH+of);

		for (int i = 0; i < Ntot; i++)
		{
			if (rand01() < 0.5)
			{
				getline(if1, temp);
				ofile << temp << " 0" << endl;
			}
			else
			{
				getline(if2, temp);
				ofile << temp << " 1" << endl;
			}
		}

		ofile.close();;
		if1.close();
		if2.close();
	}

	static void bitmap(int width, int height, int* R, int* G, int *B, string bitmapfilename)
	{
		typedef struct                       /**** BMP file header structure ****/
		{
			unsigned int   bfSize;           /* Size of file */
			unsigned short bfReserved1;      /* Reserved */
			unsigned short bfReserved2;      /* ... */
			unsigned int   bfOffBits;        /* Offset to bitmap data */
		} BITMAPFILEHEADER;

		typedef struct                       /**** BMP file info structure ****/
		{
			unsigned int   biSize;           /* Size of info header */
			int            biWidth;          /* Width of image */
			int            biHeight;         /* Height of image */
			unsigned short biPlanes;         /* Number of color planes */
			unsigned short biBitCount;       /* Number of bits per pixel */
			unsigned int   biCompression;    /* Type of compression to use */
			unsigned int   biSizeImage;      /* Size of image data */
			int            biXPelsPerMeter;  /* X pixels per meter */
			int            biYPelsPerMeter;  /* Y pixels per meter */
			unsigned int   biClrUsed;        /* Number of colors used */
			unsigned int   biClrImportant;   /* Number of important colors */
		} BITMAPINFOHEADER;

		BITMAPFILEHEADER bfh;
		BITMAPINFOHEADER bih;

		/* Magic number for file. It does not fit in the header structure due to alignment requirements, so put it outside */
		unsigned short bfType = 0x4d42;
		bfh.bfReserved1 = 0;
		bfh.bfReserved2 = 0;
		bfh.bfSize = 2 + sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + 640 * 480 * 3;
		bfh.bfOffBits = 0x36;

		bih.biSize = sizeof(BITMAPINFOHEADER);
		bih.biWidth = width;
		bih.biHeight = height;
		bih.biPlanes = 1;
		bih.biBitCount = 24;
		bih.biCompression = 0;
		bih.biSizeImage = 0;
		bih.biXPelsPerMeter = 5000;
		bih.biYPelsPerMeter = 5000;
		bih.biClrUsed = 0;
		bih.biClrImportant = 0;



		FILE *file = fopen((PATH+bitmapfilename).c_str(), "wb");
		if (!file)
		{
			printf("Could not write file\n");
			return;
		}

		/*Write headers*/
		fwrite(&bfType, 1, sizeof(bfType), file);
		fwrite(&bfh, 1, sizeof(bfh), file);
		fwrite(&bih, 1, sizeof(bih), file);

		int i = 0;
		/*Write bitmap*/
		for (int y = bih.biHeight - 1; y >= 0; y--) /*Scanline loop backwards*/
		{
			for (int x = 0; x < bih.biWidth; x++) /*Column loop forwards*/
			{
				/*compute some pixel values*/
				unsigned char r = R[i];
				unsigned char g = G[i];
				unsigned char b = B[i];
				fwrite(&b, 1, 1, file);
				fwrite(&g, 1, 1, file);
				fwrite(&r, 1, 1, file);
				i++;
			}
		}
		fclose(file);
	}

	static void generatePatternBasedDataset(int Nvv, int N, int *Nc, float prob, string filename)
	{
		ofstream ofs(filename);
		int ***pattern = new int**[2];
		pattern[0] = new int*[Nc[0]];
		for (int i = 0; i < Nc[0]; i++)
		{
			pattern[0][i] = new int[Nvv];
			for (int j = 0; j < Nvv; j++)
			{
				pattern[0][i][j] = 0;
				if (rand01() < 0.5) pattern[0][i][j] = 1;
			}
		}
		pattern[1] = new int*[Nc[1]];
		for (int i = 0; i < Nc[1]; i++)
		{
			pattern[1][i] = new int[Nvv];
			for (int j = 0; j < Nvv; j++)
			{
				pattern[1][i][j] = 0;
				if (rand01() < 0.5) pattern[1][i][j] = 1;
			}
		}
		for (int i = 0; i < N; i++)
		{
			if ((i % 10000) == 0)
				cout << "i: " << i << endl;
			int c = 0;
			if (rand01() < 0.5) c = 1;
			int pat = randInt(Nc[c]);
			for (int j = 0; j < Nvv; j++)
			{
				int bit = pattern[c][pat][j];
				if (rand01() < prob) bit = (bit + 1) % 2;
				ofs << bit << " ";
			}
			ofs << c << endl;
		}
		ofs.close();
	}
};











class Dataset
{
private:

	float **wholeDataSet;

	int N;

	int D;

	int d1, d2;

	int currentN;

	float **minibatch;

	int Msize;

	ifstream ifs;

	bool stream;

	bool binary_file;

	int scanN(string filename)
	{
		ifs.open(PATH+filename);
		int i = 0;
		string temp;
		while (!ifs.eof())
		{
			getline(ifs, temp);
			i++;
		}
		ifs.close();
		return i;
	}

	void stringToData(string temp, float* tab)
	{
		string *temp_table = Tools::splitString(temp);
		for (int j = 0; j < D; j++)
			tab[j] = stof(temp_table[j]);
	}

	void openMnist(string filename)
	{
		unsigned char ch;
		unsigned char ch2, ch3, ch4;
		ifs.open(PATH + filename, ios::in | ios::binary);
		ifs.read((char*)&ch, sizeof(ch));
		ifs.read((char*)&ch, sizeof(char));
		ifs.read((char*)&ch, sizeof(char));
		ifs.read((char*)&ch, sizeof(char));

		ifs.read((char*)&ch, sizeof(char));
		ifs.read((char*)&ch2, sizeof(char));
		ifs.read((char*)&ch3, sizeof(char));
		ifs.read((char*)&ch4, sizeof(char));

		N = Tools::mergeHexadecimals(ch, ch2, ch3, ch4);

		ifs.read((char*)&ch, sizeof(char));
		ifs.read((char*)&ch2, sizeof(char));
		ifs.read((char*)&ch3, sizeof(char));
		ifs.read((char*)&ch4, sizeof(char));

		d1 = Tools::mergeHexadecimals(ch, ch2, ch3, ch4);

		ifs.read((char*)&ch, sizeof(char));
		ifs.read((char*)&ch2, sizeof(char));
		ifs.read((char*)&ch3, sizeof(char));
		ifs.read((char*)&ch4, sizeof(char));

		d2 = Tools::mergeHexadecimals(ch, ch2, ch3, ch4);

		D = d1*d2;

	}

	void readOneMnistData(float *tab)
	{
		unsigned char ch;
		for (int j = 0; j < D; j++)
		{
			ifs.read((char*)&ch, sizeof(ch));
			tab[j] = ((float)ch) / (255.0);
		}
	}

	void copyFileToDatasetMnist(string filename)
	{
		openMnist(filename);
		wholeDataSet = new float*[N];
		for (int i = 0; i < N; i++)
		{
			if (((i+1) % 10000) == 0) cout << "MNIST dataset read, filename: " << filename << ", i: " << i << endl;
			wholeDataSet[i] = new float[D];
			readOneMnistData(wholeDataSet[i]);
		}
		ifs.close();
	}

	void copyFileToDataset(string filename)
	{
		if (binary_file)
			copyFileToDatasetMnist(filename);
		else
		{
			ifs.open(filename);
			wholeDataSet = new float*[N];
			string temp;

			getline(ifs, temp);
			D = Tools::size(temp);

			for (int i = 0; i < N; i++)
				wholeDataSet[i] = new float[D];

			stringToData(temp, wholeDataSet[0]);

			for (int i = 1; i < N; i++)
			{
				getline(ifs, temp);
				stringToData(temp, wholeDataSet[i]);
			}
			ifs.close();
		}
	}

	

	int setMinibatchFromFile(int M)
	{
		string temp;
		int m = 0;
		minibatch = new float*[M];
		for (int i = 0; i < M; i++)
		{
			if (ifs.eof()) break;
			minibatch[i] = new float[D];
			if (!binary_file)
			{
				temp = "";
				while(temp=="")	
					getline(ifs, temp);
				stringToData(temp, minibatch[i]);
			}
			else
				readOneMnistData(minibatch[i]);
			m++;
		}
		Msize = m;
		return m;
	}

	int setMinibatchFromData(int M, bool random)
	{
		int m = 0;
		minibatch = new float*[M];
		for (int i = 0; i < M; i++)
		{
			if (random) 
			{
				minibatch[i] = wholeDataSet[(Tools::randInt(N))]; 
				m++;
			}
			else
			{
				minibatch[i] = wholeDataSet[(currentN + i) % N];
				m++;
			}
		}
		if (!random) currentN += m;
		currentN = currentN % N;
		Msize = m;
		return m;
	}
	

	void openFile(string filename)
	{
		if (!binary_file)
		{
			string temp;
			ifs.open(PATH+filename);
			getline(ifs, temp);
			D = Tools::size(temp);
			ifs.close();
			ifs.open(PATH+filename);
		}
		else
			openMnist(filename);
	}


public:

	Dataset(bool str, string filename)
	{
		binary_file = false;
		string* extens = Tools::splitString(filename);
		if (((*(extens + 1)) == "udx3-ubyte") || ((*(extens + 1)) == "udx1-ubyte"))
			binary_file = true;
		currentN = 0;
		wholeDataSet = NULL;
		minibatch = NULL;
		stream = str;
//		binary_file = binary;
		if (!stream)
		{
			if (!binary_file)
				N = scanN(filename);
			copyFileToDataset(filename);
		}
		else
			openFile(filename);
	}

	~Dataset()
	{
		if (stream)
		{
			ifs.close();
			for (int i = 0; i < Msize; i++)
				delete minibatch[i];
			delete minibatch;
		}
		else
		{
			for (int i = 0; i < N; i++)
				delete wholeDataSet[i];
			delete wholeDataSet;
			if (minibatch != NULL)
				delete minibatch;
		}
	}

	int setMinibatch(int M, bool rand)
	{
		if (stream) return setMinibatchFromFile(M);
		else return setMinibatchFromData(M, rand);
	}

	float* getDataElement(int i) { return minibatch[i]; }

	float getDataElement(int i, int j) { return minibatch[i][j]; }

	int getD() { return D; }

	int getM() { return Msize; }

	int getN() { return N; }

};










class SimpleLayer
{
private:

	int size;

	int ID;

	int startMet;

	int startBMet;

	float* neur;

	float **persistent_neurons;
	int persistent_size;

	float* bias;

	int type;

	void remove_persistent_layers()
	{
		for (int i = 0; i < persistent_size; i++)
			delete persistent_neurons[i];
		delete persistent_neurons;
	}

	void setRandomLayer(float *table) { Tools::setRandomValues(size, table, startMet); }

public:

	SimpleLayer(int id, int s, int sm, int sbm, int t) 
	{
		ID = id;

		size = s;

		startMet = sm;

		startBMet = sbm;

		type = t;

		neur = new float[s];
		bias = new float[s];

		Tools::setRandomValues(size, neur, startMet);
		Tools::setRandomValues(size, bias, startBMet);

	};

	SimpleLayer(int id, int s, int sm, int sbm, int t, int ps)
		: SimpleLayer(id, s, sm, sbm, t)
	{
		initPersistents(ps);
	}

	~SimpleLayer()
	{
		delete neur;
		delete bias;
		if (persistent_neurons != NULL)
			remove_persistent_layers();
	}

	int getSize() { return size; }

	int getID() { return ID; }

	int getType() { return type; }

	int getStartMet() { return startMet; }

	int getStartBMet() { return startBMet; }

	float* getNeuron() { return neur; }

	float getNeuron(int i) { return neur[i]; }

	float getNeuron(int i, int ps)
	{
		if (ps >= 0)
			return persistent_neurons[ps][i];
		else if (ps == -1)
			return getNeuron(i);
		return 0.0;
	}

	float* getBias() { return bias; }

	float getBias(int i) { return bias[i]; }

	void putNeuron(float* nr) { neur = nr; }

	void putNeuron(int i, float nr) { *(neur + i) = nr; }

	void putNeuron(int i, float nr, int ps)
	{
		if (ps >= 0)
			*(persistent_neurons[ps] + i) = nr;
		else if (ps == -1)
			putNeuron(i, nr);
	}

	void putRandomLayer() 	{ setRandomLayer(neur); }

	void putRandomPersistentLayer(int pi) { setRandomLayer(persistent_neurons[pi]); }

	void setBias(float* bs) { bias = bs; }

	void setBias(int i, float bs) { *(bias + i) = bs; }

	void addBias(int i, float bs) { *(bias + i) += bs; }

	void addBias(float* bs)
	{
		for (int i = 0; i < size; i++)
			addBias(i, *(bs + i));
	}

	void initPersistents(int ps)
	{
		persistent_size = ps;
		persistent_neurons = new float*[persistent_size];
		for (int i = 0; i < persistent_size; i++)
		{
			persistent_neurons[i] = new float[size];
			Tools::setRandomValues(size, persistent_neurons[i], startMet);
		}
	}

	int getPersistentSize() { return persistent_size; }

};










class SimpleSynapses
{
private:

	SimpleLayer* slayer1;
	SimpleLayer* slayer2;

	float *weight;

	int size1;
	int size2;

	int startWMet;

	int getWeightNumber(int i, int j) { return i*size2 + j; }
	
	void splitWeightNumber(int w, int* i, int* j)
	{
		*i = w / size2;
		*j = w % size2;
	}


public:

	SimpleSynapses(SimpleLayer* sl1, SimpleLayer* sl2, int startWeightMethod)
	{
		slayer1 = sl1;
		slayer2 = sl2;

		size1 = slayer1->getSize();
		size2 = slayer2->getSize();

		startWMet = startWeightMethod;

		weight = new float[size1*size2];

		for (int i = 0; i < size1; i++)
			for (int j = 0; j < size2; j++)
			{
				weight[getWeightNumber(i, j)] = 0.0;
				if (startWMet == 1)
					weight[getWeightNumber(i, j)] = Tools::gaussian(0.0, 1.0 / ((float)(size2)));
			}
	}

	~SimpleSynapses() { delete weight; }

	int getID(int i) { return (i == 0) ? slayer1->getID() : slayer2->getID(); }

	int getSize(int i) { return (i == 0) ? slayer1->getSize() : slayer2->getSize(); }

	int getStartWMet() { return startWMet; }

	float* getWeight() { return weight; }

	float getWeight(int i) { return weight[i]; }

	float getWeight(int i, int j) { return weight[getWeightNumber(i, j)]; }

	void setWeight(float *w) { weight = w; }

	void setWeight(int i, float w) { *(weight + i) = w; }

	void setWeight(int i, int j, float w) { *(weight + getWeightNumber(i, j)) = w; }

	void addWeight(int i, float w) { *(weight + i) += w; }

	void addWeight(int i, int j, float w) { *(weight + getWeightNumber(i, j)) = +w; }

};










class NeuralNetwork
{
private:

	int NofLayers;

	float* momentum;
	bool momentum_enabled;

	float friction;

	float learning_rate;

	int learning_strategy;

	int minibatch_size;

	int method;

	int K;

	SimpleLayer** layers;

	SimpleSynapses** synapses;

//	void(NeuralNetwork::*learning_1ex)(int, int, float*);

	void setParameters(int nol, int* lsizes, int* ltypes, int* lsms, int* swsms, int* sbsms)
	{
		NofLayers = nol;

		layers = new SimpleLayer*[NofLayers];

		for (int i = 0; i < NofLayers; i++)
				layers[i] = new SimpleLayer(i, (*(lsizes + i)), (*(lsms + i)), (*(sbsms + i)), (*(ltypes + i)));

		synapses = new SimpleSynapses*[NofLayers - 1];

		for (int i = 0; i < NofLayers - 1; i++)
			synapses[i] = new SimpleSynapses(layers[i], layers[i + 1], (*(swsms + i)));
	}

	void loadNeuralNetwork(string filename)
	{
		ifstream ifile;
		ifile.open(filename);

		string temp;
		string* temptable;
		int ns = 0;
		int s = 0;

		bool OK = true;

		while (!ifile.eof())
		{
			getline(ifile, temp);
			s = Tools::size(temp);
			int desired_size = ((ns % 2) == 1) ? synapses[ns/2]->getSize(0)*synapses[ns/2]->getSize(1) : layers[ns/2]->getSize();
			if (s != desired_size)
			{
				if((ns % 2) == 1)
					cout << "imcompatible size of synapse group " << ns/2 << " between layers " << ns/2 << " and " << (ns/2) + 1 << endl;
				else
					cout << "imcompatible size of layer " << ns / 2 << endl;
				OK = false;
			}
			ns++;
		}

		if (((ns/2) + 1) != NofLayers)
		{
			cout << "incompatible number of layers or synapses groups" << endl;
			OK = false;
		}

		ifile.close();

		ns = 0;
		if (OK)
		{
			ifile.open(filename);
			while (!ifile.eof())
			{
				getline(ifile, temp);
				s = Tools::size(temp);
				temptable = Tools::splitString(temp);
				for (int j = 0; j < s; j++)
				{
					if ((ns % 2) == 1)
						synapses[ns / 2]->setWeight(j, stof(*(temptable + j)));
					else
						layers[ns / 2]->setBias(j, stof(*(temptable + j)));
				}
				ns++;
			}
			ifile.close();
		}
	}

	void saveNeuralNetworkConfig(string filename)
	{
		
		ofstream ofile(filename);
	
		ofile << NofLayers << endl;
		
		for (int i = 0; i < (NofLayers - 1); i++)
			ofile << layers[i]->getSize() << " ";
		ofile << layers[NofLayers - 1]->getSize() << endl;

		for (int i = 0; i < (NofLayers - 1); i++)
			ofile << layers[i]->getType() << " ";
		ofile << layers[NofLayers - 1]->getType() << endl;
		
		for (int i = 0; i < (NofLayers - 1); i++)
			ofile << layers[i]->getStartMet() << " ";
		ofile << layers[NofLayers - 1]->getStartMet() << endl;

		for (int i = 0; i < (NofLayers - 2); i++)
			ofile << synapses[i]->getStartWMet() << " ";
		ofile << synapses[NofLayers - 2]->getStartWMet() << endl;

		for (int i = 0; i < (NofLayers - 1); i++)
			ofile << layers[i]->getStartBMet() << " ";
		ofile << layers[NofLayers - 1]->getStartBMet() << endl;

		ofile.close();
	}

	void saveNeuralNetworkWeights(string filename)
	{
		ofstream ofile;
		ofile.open(filename);

		for (int i = 0; i < (NofLayers - 1); i++)
		{
			int szsz = layers[i]->getSize();
			for (int j = 0; j < (szsz - 1); j++)
				ofile << layers[i]->getBias(j) << " ";
			ofile << layers[i]->getBias(szsz - 1) << endl;
			szsz = synapses[i]->getSize(0)*synapses[i]->getSize(1);
			for (int j = 0; j < (szsz - 1); j++)
				ofile << synapses[i]->getWeight(j) << " ";
			ofile << synapses[i]->getWeight(szsz - 1) << endl;
		}
		int szsz = layers[NofLayers-1]->getSize();
		for (int j = 0; j < (szsz - 1); j++)
			ofile << layers[NofLayers-1]->getBias(j) << " ";
		ofile << layers[NofLayers-1]->getBias(szsz - 1) << endl;

		ofile.close();
	}

	float getSignalForward(int lay, int neu, int ps)
	{
		float ret = 0.0;
		if ((lay > 0) && (lay < NofLayers))
		{
			for (int i = 0; i < layers[lay - 1]->getSize(); i++)
			{
				ret = ret + layers[lay - 1]->getNeuron(i, ps) * synapses[lay - 1]->getWeight(i, neu);
			}
			ret = ret + layers[lay]->getBias(neu);
		}
		return ret;
	}

	float getSignalBackward(int lay, int neu, int ps)
	{
		float ret = 0.0;
		if ((lay < (NofLayers - 1)) && (lay >= 0))
		{
			for (int i = 0; i < layers[lay + 1]->getSize(); i++)
			{
				ret = ret + layers[lay+1]->getNeuron(i, ps) * synapses[lay]->getWeight(neu, i);
	//			cout << "neur: " << layers[lay]->getNeuron(neu, ps) << ", synaps: " << synapses[lay]->getWeight(neu, i) << ", ret: " << ret << endl;
			}
			ret = ret + layers[lay]->getBias(neu);
//			cout << "bias: " << layers[lay]->getBias(neu) << ", ret: " << ret << endl;
		}
		return ret;
	}

	float getSignal(int lay, int neu, bool forw, int ps)
	{
		if (forw) return getSignalForward(lay, neu, ps);
		else
			return getSignalBackward(lay, neu, ps);
	}

	float getSignal(int lay, int neu, bool forw) { return getSignal(lay, neu, forw, -1); }

	float sigmoid(float val) { return 1.0 / (1.0 + expf(-val)); }

	float activate(float val) { return ((Tools::rand01() < val) ? 1.0 : 0.0); }

	void activate(int lay, int neu, int ps) { ((Tools::rand01() < layers[lay]->getNeuron(neu, ps)) ? 
		layers[lay]->putNeuron(neu,1.0, ps) : layers[lay]->putNeuron(neu, 0.0, ps)); }

	void activate(int lay, int ps)
	{
		for (int i = 0; i < layers[lay]->getSize(); i++)
			activate(lay, i, ps);
	}

	void activate(int lay) { activate(lay, -1); }

	float getCorrelationW(int lay, int i, int j, int ps) { return layers[lay - 1]->getNeuron(i, ps)*layers[lay]->getNeuron(j, ps); }

	float getCorrelationW(int lay, int w, int ps) { return getCorrelationW(lay, w / layers[lay]->getSize(), w % layers[lay]->getSize(), ps); }
	
	float getCorrelationB(int lay, int b, int ps) { return layers[lay]->getNeuron(b, ps); }

	void getCorrelation(int lay, float* grads, float multiply, int ps)
	{
		int s1 = layers[lay - 1]->getSize();
		int s2 = layers[lay]->getSize();
		for (int i = 0; i < s1; i++)
			(*(grads + i)) += multiply*getCorrelationB(lay - 1, i, ps);
		float* grads2 = grads + s1;
		for (int i = 0; i < s1*s2; i++)
			(*(grads2 + i)) += multiply*getCorrelationW(lay, i, ps);
		grads2 = grads2 + s1*s2;
		for (int i = 0; i < s2; i++)
			(*(grads2 + i)) += multiply*getCorrelationB(lay, i, ps);
	}

	void getCorrelation(int lay, float* grads, float multiply) { getCorrelation(lay, grads, multiply, -1); }

public:

	NeuralNetwork(int nol, int* lsizes, int* ltypes, int* lsms, int* swsms, int* sbsms)
	{
		setParameters(nol, lsizes, ltypes, lsms, swsms, sbsms);
	}
		
	NeuralNetwork(string filename)
	{
		string temp;
		int n = 0;
		string* temptab;
		ifstream ifile;
		ifile.open(filename);

		int** a = new int*[6];

		a[0] = new int[1];
		getline(ifile, temp);
		if (Tools::size(temp) != 1)
			*a[0] = 0;
		else
			*a[0] = stoi(temp);

		for (int i = 1; i < 6; i++)
		{
			a[i] = new int[*a[0]];
			getline(ifile, temp);
			n = Tools::size(temp);
			temptab = Tools::splitString(temp);
			int ndesired = (*a[0]) - (i>3);
			for (int j = 0; j < ndesired; j++)
			{
				if (n != ndesired)
					*(a[i] + j) = stoi(*temptab);
				else
					*(a[i] + j) = stoi(*(temptab + j));
			}
		}

		setParameters(*a[0], a[1], a[2], a[3], a[4], a[5]);

		ifile.close();
	}

	NeuralNetwork(string filenameconfig, string filenamenetwork): NeuralNetwork(filenameconfig)
	{
		loadNeuralNetwork(filenamenetwork);
	}

	void saveNeuralNetwork(string filenameconfig, string filenamenetwork)
	{
		saveNeuralNetworkConfig(filenameconfig);
		saveNeuralNetworkWeights(filenamenetwork);
	}

	void activate(int lay, int neu, bool forw, bool meanF, int ps)
	{
		float sig = getSignal(lay, neu, forw, ps);
		sig = sigmoid(sig);
		if (!meanF)
			sig = activate(sig);
		layers[lay]->putNeuron(neu, sig, ps);
	}

	void activate(int lay, bool forw, bool meanF, int ps)
	{
		for (int i = 0; i < layers[lay]->getSize(); i++)
			activate(lay, i, forw, meanF, ps);
	}

	void activate(int lay, bool forw, bool meanF) { activate(lay, forw, meanF, -1); }

	void positive_phase(int lay, bool mF, float* grads) 
	{ 
		activate(lay, true, mF); 
	//	cout << "after activation in positive phase: " << endl;
	//	plotNet();
		getCorrelation(lay, grads, 1.0);
		if (mF)
			activate(lay);
	}

	void Gibbs_Sampling(int k, int lay, bool start, bool mFrecon, float* grads, int ps)
	{
		if (start)
			activate(lay, true, false, ps);
		for (int i = 0; i < k; i++)
		{
			activate(lay - 1, false, mFrecon, ps);
	//		cout << "after reconstruction in Gibbs sampling: " << endl;
	//		plotNet();
			activate(lay, true, (i == (k-1))? true : false, ps);
	//		cout << "after reactvation from reconstruction: " << endl;
	//		plotNet();
		}	
		double mul = (ps == (-1)) ? 1.0 : -1.0 / ((float)(layers[lay]->getPersistentSize()));
		getCorrelation(lay, grads, -mul);
		activate(lay, ps);
	}

	void Gibbs_Sampling(int k, int lay, bool start, bool mFrecon, float* grads) { Gibbs_Sampling(k, lay, start, mFrecon, grads, -1); }

	void Contrastive_Divergence_1ex(int k, int lay, float* grads)
	{
		positive_phase(lay, true, grads);
//		cout << "after positive phase:" << endl;
//		plotNet();
		Gibbs_Sampling(k, lay, false, true, grads);
	}

	void Gibbs_Sampling_1ex(int k, int lay, float* grads) 
	{ 
		positive_phase(lay, true, grads);
		layers[lay - 1]->putRandomLayer();
		Gibbs_Sampling(k, lay, true, true, grads); }

	void Persistent_Contrastive_Divergence_1ex(int k, int lay, float* grads)
	{
		positive_phase(lay, true, grads);
		for (int ps = 0; ps < layers[lay]->getPersistentSize(); ps++)
			Gibbs_Sampling(k, lay, true, true, grads, ps);
	}

	void updateWeights(int lay, float* grads)
	{
		int s1 = layers[lay - 1]->getSize();
		int s2 = layers[lay]->getSize();
		for (int i = 0; i < s1; i++)
			layers[lay - 1]->addBias(i, -(*(grads + i)));
		float* grads2 = grads + s1;
		for (int i = 0; i < s1*s2; i++)
			synapses[lay - 1]->addWeight(i, -(*(grads2 + i)));
		grads2 = grads2 + s1*s2;
		for (int i = 0; i < s2; i++)
			layers[lay]->addBias(i, -(*(grads2 + i)));
	}

	void updateWeights_momentum(int lay, float *grads)
	{
		int s1 = layers[lay - 1]->getSize();
		int s2 = layers[lay]->getSize();
		int BigSize = s1 + s1*s2 + s2;
		for (int i = 0; i < BigSize; i++)
		{
			(*(momentum + i)) = friction*(*momentum + i) + (*(grads+i));
		}
		updateWeights(lay, momentum);
	}

	void update(int lay, float* grads)
	{
		if (momentum_enabled)
			updateWeights_momentum(lay, grads);
		else
			updateWeights(lay, grads);
	}

	float reconstruction_error(Dataset *ds, int i, bool bitmapa)
	{
		int d = ds->getD();
		for (int j = 0; j < d; j++)
			layers[0]->putNeuron(j, ds->getDataElement(i, j));
//		if (bitmapa) generateBitmap(1, 28 * 28, "original.bmp");
		activate(1, true, false);
		activate(0, false, true);
//		if (bitmapa) generateBitmap(1, 28 * 28, "reconstructed.bmp");
		float ret = 0.0;
		for (int j = 0; j < d; j++)
			ret = ret + sqrtf(powf(ds->getDataElement(i,j) - layers[0]->getNeuron(j),2.0));
		return ret; 
	}

	float reconstruction_error(Dataset *ds, bool bmp)
	{
		float ret = 0.0;
		int M = ds->getM();
		for (int i = 0; i < M; i++)
			ret = ret + reconstruction_error(ds, i, bmp);
		return ret / ((float)M);
	}

	float free_energy(Dataset *ds, int i)
	{
		float ret = 0.0;
		int d = ds->getD();
		for (int j = 0; j < d; j++)
		{
			layers[0]->putNeuron(j, ds->getDataElement(i, j));
			ret = ret - layers[0]->getNeuron(j)*layers[0]->getBias(j);
		}
		for (int k = 0; k < layers[1]->getSize(); k++)
		{
			float sig = getSignal(1, k, true);
			if (sig > 88.0)
				ret = ret - sig;
			else
				ret = ret - logf(1.0 + expf(sig));
		}
		return ret;
	}

	float free_energy(Dataset *ds)
	{
		float ret = 0.0;
		int M = ds->getM();
		for (int i = 0; i < M; i++)
			ret = ret + free_energy(ds, i);
		return ret / ((float)M);
	}
	
	void LearnMinibatch(Dataset* dset, int m, int d)
	{
		int s1 = layers[0]->getSize();
		int s2 = layers[1]->getSize();
		float *grds = new float[s1 + s1*s2 + s2];
		for (int i = 0; i < s1 + s1*s2 + s2; i++)
			grds[i] = 0.0;
		
//		if (method == 0) learning_1ex = Contrastive_Divergence_1ex;
//		if (method == 1) learning_1ex = Gibbs_Sampling_1ex;
//		if (method == 2) learning_1ex = Persistent_Contrastive_Divergence_1ex;

		

		if(method == 0)
			for (int i = 0; i < m; i++)
			{
				
				for (int j = 0; j < d; j++)
					layers[0]->putNeuron(j,dset->getDataElement(i,j));
		//		cout << "after data put" << endl;
		//		plotNet();
				Contrastive_Divergence_1ex(K, 1, grds);
		//		cout << "after CD" << endl;
		//		plotNet();
			}

		if (method == 1)
			for (int i = 0; i < m; i++)
			{
				for (int j = 0; j < d; j++)
					layers[0]->putNeuron(j, dset->getDataElement(i, j));
				Gibbs_Sampling_1ex(K, 1, grds);
			}

		if (method == 2)
			for (int i = 0; i < m; i++)
			{
				for (int j = 0; j < d; j++)
					layers[0]->putNeuron(j, dset->getDataElement(i, j));
				Persistent_Contrastive_Divergence_1ex(K, 1, grds);
			}

		for (int i = 0; i < s1 + s1*s2 + s2; i++)
			*(grds + i) = -learning_rate*(*(grds + i)) / ((float)(m));

		update(1, grds);

	//	cout << endl << "after update (inside): " << endl;
	//	plotNet();
	}

	void setLearningParameters(float lr, int ls, double fric, int mini, int meth, int k, int ps)
	{
		learning_rate = lr;

		learning_strategy = ls;

		momentum_enabled = false;
		if (friction != 0.0)
		{
			friction = fric;
			momentum_enabled = true;
			int s1 = layers[0]->getSize();
			int s2 = layers[1]->getSize();
			momentum = new float[s1 + s1*s2 + s2];
			for (int i = 0; i < (s1 + s1*s2 + s2); i++)
				momentum[i] = 0.0;
		}

		minibatch_size = mini;

		method = meth;
		if (method == 2)
			layers[0]->initPersistents(ps);

		K = k;
	}

	void Learn(Dataset *ds, bool rand, bool free, bool recon, int Niter, bool lea, ofstream &ofs, int startn)
	{
		float lambda = 0.998;

		float ii = 0.0;
		float ii2 = 0.0;

		float recav = 0.0;
		float freeav = 0.0;

		float recii = 0.0;
		float freeii = 0.0;

		float nn = 0.0;


//		ofstream ofs;
		float rec = 0.0;
		float freeE = 0.0;
		bool any = ((free) || (recon));
		bool all = ((free) && (recon));
//		if (any)
//			ofs.open("evaluation.txt");
		double lear = learning_rate;
		for (int i = startn; i < (startn+Niter); i++)
		{
			bool show = ((i % 10) == 0);
			int m = ds->setMinibatch(minibatch_size, rand);
			int d = ds->getD();

			if (any)
			{
				ofs << i << " ";
				if (show)
					cout << i << " ";
			}
			if (free)
			{
				freeE = free_energy(ds);
				ofs << freeE;
				if (show)
					cout << ", free: " << freeE;
			}
			if (all)
			{
				ofs << " ";
				if (show)
					cout << " ";
			}
			if (recon)
			{
				rec = reconstruction_error(ds, ((i + 100) % 200) == 0);
				ofs << rec;
				if (show)
					cout << ", rec: " << rec;
			}
	//		if ((i % 1) == 0)
				//		cout << " " << averageWeight(1) << ", " << averageAbsWeight(1) << ", " << averageAbsBias(0) << 
				//			", " << averageAbsBias(1) << ", " << maxAbsWeight(1) << "..";
				if (any)
				{
					ii = ii*lambda + (i - startn);
					ii2 = ii2*lambda + (i - startn)*(i - startn);
					recav = recav*lambda + rec;
					freeav = freeav*lambda + freeE;
					recii = recii*lambda + (i - startn)*rec;
					freeii = freeii*lambda + (i - startn)*freeE;
					nn = nn*lambda + 1.0;

					float regresrec = (nn*recii - recav*ii) / (nn*ii2 - ii*ii);
					float regresfree = (nn*freeii - freeav*ii) / (nn*ii2 - ii*ii);

					ofs << " " << regresrec << " " << regresfree;


					ofs << endl;
					if (show)
						cout << endl;
				}

	//		learning_rate = lear / powf(((float)(i + 1)),0.33);


				

			if(lea)
				LearnMinibatch(ds, m, d);
			
	//		cout << "after update, i: " << i << endl;
	//		plotNet();
			
		}
	//	if(ofs.is_open())
	//		ofs.close();
	}

	void Learn(Dataset *ds, bool free, bool recon, int Niter, bool lea, ofstream &ofs, int startn) { Learn(ds, false, free, recon, Niter, lea, ofs, startn); }

	float averageWeight(int lay)
	{
		float ret = 0.0;
		int s1 = layers[0]->getSize();
		int s2 = layers[1]->getSize();
		for (int i = 0; i < s1*s2; i++)
			ret = ret + synapses[lay - 1]->getWeight(i);
		return ret / ((float)(s1*s2));
	}

	float averageAbsWeight(int lay)
	{
		float ret = 0.0;
		int s1 = layers[lay-1]->getSize();
		int s2 = layers[lay]->getSize();
		for (int i = 0; i < s1*s2; i++)
			ret = ret + abs(synapses[lay - 1]->getWeight(i));
		return ret / ((float)(s1*s2));
	}

	float averageAbsBias(int lay)
	{
		float ret = 0.0;
		int s1 = layers[lay]->getSize();
		for (int i = 0; i < s1; i++)
			ret = ret + abs(layers[lay]->getBias(i));
		return ret / ((float)(s1));
	}

	float maxAbsWeight(int lay)
	{
		float ret = synapses[lay - 1]->getWeight(0);
		int s1 = layers[lay - 1]->getSize();
		int s2 = layers[lay]->getSize();
		for (int i = 1; i < s1*s2; i++)
			if (abs(synapses[lay - 1]->getWeight(i)) > ret)
				ret = abs(synapses[lay - 1]->getWeight(i));
		return ret;
	}

	void plotNet()
	{
		int s1 = layers[0]->getSize();
		for (int i = 0; i < s1; i++)
			cout << layers[0]->getNeuron(i) << " ";
		cout << endl;
		for (int i = 0; i < s1; i++)
			cout << layers[0]->getBias(i) << " ";
		cout << endl;
		cout << endl;
		int s2 = layers[1]->getSize();
		for (int i = 0; i < s1*s2; i++)
			cout << synapses[0]->getWeight(i) << " ";
		cout << endl;
		cout << endl;
		for (int i = 0; i < s2; i++)
			cout << layers[1]->getNeuron(i) << " ";
		cout << endl;
		for (int i = 0; i < s2; i++)
			cout << layers[1]->getBias(i) << " ";
		cout << endl;
		cout << endl;
		cout << endl;
		cin.get();
	}

	void generateBitmap(int scale, int size, string filename)
	{
		int* bitmp = new int[size*scale];
		for (int i = 0; i < size*scale; i++)
			bitmp[i] = (int)(255.0*layers[0]->getNeuron(i / scale));
		Tools::bitmap(28, 28, bitmp, bitmp, bitmp, filename);
	}
	
};

















cudaError_t addWithCuda(int *c, const int *a, const int *b, unsigned int size);

__global__ void addKernel(int *c, const int *a, const int *b)
{
    int i = threadIdx.x;
    c[i] = a[i] + b[i];
}

int main()
{
	int a;

	srand(time(NULL));

//	Tools::makeCDFile("energy_dataset1.txt", "energy_dataset2.txt", "gradualdrift_energy.txt", 800000, 500000, 600000, false);
//	Tools::makeCDFile("pattern_dataset1.txt", "pattern_dataset2.txt", "suddendrift_pattern.txt", 120000, 100000, 100000, false);
//	Tools::makeCDFile("train-images0247.idx3-ubyte", "train-images135689.idx3-ubyte", "suddendrift.idx3-ubyte", 59800, 23800, 36000, true);

//	ifstream iff(PATH+"suddendrift.idx3-ubyte", ios::in | ios::binary);

/*	unsigned char ch;

	for (int i = 0; i < 110; i++)
	{
		iff.read((char*)&ch, sizeof(char));
		cout << (int)ch << endl;
	}
*/

//	Tools::bitmap(300, 100, NULL, NULL, NULL, "bitmapa.bmp");

	//Tools::generateEnergyBasedDataset(20, 2000000, "energy_dataset1.txt");
//	Tools::generatePatternBasedDataset(40, 2000000, new int[2]{ 1200,1200 }, 0.15, "pattern_dataset2.txt");

	ofstream ofs("evaluation_sudd.txt");

	NeuralNetwork nn(2, new int[2]{21, 15 }, new int[2]{ 0,0 }, new int[2]{ 1,1 }, new int[1]{ 1 }, new int[2]{ 3,3 });

	

	Dataset *ds = new Dataset(true,"suddendrift_energy.txt");

	nn.setLearningParameters(0.05, 0, 0.0, 20, 0, 1, -1);

	nn.Learn(ds, true, true, 20000, true,ofs,0);

	nn.Learn(ds, true, true, 20000, false, ofs,20000);


//	Dataset *ds2 = new Dataset(true, false, "energy_dataset2.txt");

//	nn.Learn(ds2, true, true, 2000, false,ofs);
	
	cin >> a;

	ofs.close();

/*	srand(time(NULL));

    const int arraySize = 5;
    const int a[arraySize] = { 1, 2, 3, 4, 5 };
    const int b[arraySize] = { 10, 20, 30, 40, 50 };
    int c[arraySize] = { 0 };

	

    // Add vectors in parallel.
    cudaError_t cudaStatus = addWithCuda(c, a, b, arraySize);
    if (cudaStatus != cudaSuccess) {
        cout << "addWithCuda failed!";
        return 1;
    }

    cout << "{1,2,3,4,5} + {10,20,30,40,50} = {" << c[0] << "," <<
		c[1] << "," << c[2] << "," << c[3] << "," << c[4] << "}" << endl;

    // cudaDeviceReset must be called before exiting in order for profiling and
    // tracing tools such as Nsight and Visual Profiler to show complete traces.
    cudaStatus = cudaDeviceReset();
    if (cudaStatus != cudaSuccess) {
        cout << "cudaDeviceReset failed!";
        return 1;
    }
*/
    return 0;
}

// Helper function for using CUDA to add vectors in parallel.
cudaError_t addWithCuda(int *c, const int *a, const int *b, unsigned int size)
{
    int *dev_a = 0;
    int *dev_b = 0;
    int *dev_c = 0;
    cudaError_t cudaStatus;

    // Choose which GPU to run on, change this on a multi-GPU system.
    cudaStatus = cudaSetDevice(0);
    if (cudaStatus != cudaSuccess) {
        cout << "cudaSetDevice failed!  Do you have a CUDA-capable GPU installed?";
        goto Error;
    }

    // Allocate GPU buffers for three vectors (two input, one output)    .
    cudaStatus = cudaMalloc((void**)&dev_c, size * sizeof(int));
    if (cudaStatus != cudaSuccess) {
        cout << "cudaMalloc failed!";
        goto Error;
    }

    cudaStatus = cudaMalloc((void**)&dev_a, size * sizeof(int));
    if (cudaStatus != cudaSuccess) {
        cout << "cudaMalloc failed!";
        goto Error;
    }

    cudaStatus = cudaMalloc((void**)&dev_b, size * sizeof(int));
    if (cudaStatus != cudaSuccess) {
        fprintf(stderr, "cudaMalloc failed!");
        goto Error;
    }

    // Copy input vectors from host memory to GPU buffers.
    cudaStatus = cudaMemcpy(dev_a, a, size * sizeof(int), cudaMemcpyHostToDevice);
    if (cudaStatus != cudaSuccess) {
        cout << "cudaMemcpy failed!";
        goto Error;
    }

    cudaStatus = cudaMemcpy(dev_b, b, size * sizeof(int), cudaMemcpyHostToDevice);
    if (cudaStatus != cudaSuccess) {
        cout << "cudaMemcpy failed!";
        goto Error;
    }

    // Launch a kernel on the GPU with one thread for each element.
    addKernel<<<1, size>>>(dev_c, dev_a, dev_b);

    // Check for any errors launching the kernel
    cudaStatus = cudaGetLastError();
    if (cudaStatus != cudaSuccess) {
		cout << "addKernel launch failed: " << cudaGetErrorString(cudaStatus) << endl;
        goto Error;
    }
    
    // cudaDeviceSynchronize waits for the kernel to finish, and returns
    // any errors encountered during the launch.
    cudaStatus = cudaDeviceSynchronize();
    if (cudaStatus != cudaSuccess) {
        cout << "cudaDeviceSynchronize returned error code " << cudaStatus << " after launching addKernel!" << endl;
        goto Error;
    }

    // Copy output vector from GPU buffer to host memory.
    cudaStatus = cudaMemcpy(c, dev_c, size * sizeof(int), cudaMemcpyDeviceToHost);
    if (cudaStatus != cudaSuccess) {
        cout << "cudaMemcpy failed!";
        goto Error;
    }

Error:
    cudaFree(dev_c);
    cudaFree(dev_a);
    cudaFree(dev_b);
    
    return cudaStatus;
}
